library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

-- --------------------------------------------------------
--                                                       --   
--  ZX-spectrum AT keyboard interface CPLD code jrok'06  --
--                                                       --   
-- --------------------------------------------------------


entity speckey is
    Port ( addr : in std_logic_vector(7 downto 0);
           z80_db  : out std_logic_vector(4 downto 0);
           mcu_kbd 		: in std_logic ;
           mcu_clk 		: in std_logic ;
           mcu_reset 	: in std_logic;
		 z80_rd		: in std_logic;
		 z80_reset	: in std_logic;
		 iorq_ULA		: in std_logic;
		 z80_a0   	: in std_logic;
		 z80_IORQ		: in std_logic


		 );
end speckey;


-- For references
--
-- Addr(0) = Z80 a8
-- Addr(1) = Z80 a9
-- Addr(2) = Z80 a10
-- Addr(3) = Z80 a11
-- Addr(4) = Z80 a12
-- Addr(5) = Z80 a13
-- Addr(6) = Z80 a14
-- Addr(7) = Z80 a15


architecture Behavioral of speckey is

-- 40 spectrum keyboard keys
type kb_mem is  array( 0 to 39 ) of  std_logic;

 signal kb_data : kb_mem;
 signal kb_addr : integer range 0 to 39;


begin


-- ---------------------------------------
-- Read in the data from MCU's serial bus
-- ---------------------------------------

process( mcu_clk, mcu_kbd, mcu_reset)
begin

if ( mcu_reset = '1'  ) then
  kb_addr <= 0;
else

	if ( rising_edge( mcu_clk )) then
		-- read the key status from the micro-controller
		-- if the bit is '1' that means the key is pressed
  		kb_data( conv_integer(kb_addr)  ) <=  mcu_kbd;
	end if;

	if ( falling_edge( mcu_clk )) then
		-- increment the pointed to the address of the next 5
		kb_addr <= kb_addr + 1;
	end if;
 			 
end if;

end process;


-- NOTE: Reading the keyboard
-- 
-- because the ULA is "isolated" from the Z80's bus by resistors
-- then the CPLD has electrical "priority" over the ULA on the databus
-- so when the CPLD outputs the keyboard bit status the z80 sees
-- these values NOT the output from the Sinclair ULA


-- -------------------------------------
-- Spectrum is reading
-- -------------------------------------

process( iorq_ULA, z80_rd,  kb_data, addr(0), addr(1), addr(2), addr(3), addr(4), addr(5), addr(6), addr(7) )
begin

-- or z80_reset = '0'
--if (  z80_IORQ = '0' and z80_rd= '0'  and z80_a0 = '0' ) then	



if (  iorq_ULA = '0' and z80_rd= '0' ) then

-- output the data (!)
--
-- if an address line is low then set the databus to the bit value for that column
-- so if multiple address lines are low
-- the up/down status of MULTIPLE 'keybits' will be passeds
--
-- key logic is INVERTED so that pressed(down)='0' and released(up)='1'

	   
		z80_db(0) <=		not(	( kb_data(0) and not( addr(0)) ) 
					or 	( kb_data(5) and not( addr(1)) ) 
					or 	( kb_data(10) and not(addr(2) ) ) 
					or 	( kb_data(15) and not(addr(3) ) ) 
					or 	( kb_data(20) and not(addr(4) ) ) 
					or 	( kb_data(25) and not(addr(5) ) ) 
					or 	( kb_data(30) and not(addr(6) ) ) 
					or 	( kb_data(35) and not(addr(7) ) )  );

		z80_db(1) <=		not(	( kb_data(1)  and not( addr(0)) ) 
					or   ( kb_data(6)  and not( addr(1)) ) 
					or   ( kb_data(11) and not(addr(2)) ) 
					or   ( kb_data(16) and not(addr(3)) ) 
					or   ( kb_data(21) and not(addr(4)) ) 
					or   ( kb_data(26) and not(addr(5)) ) 
					or   ( kb_data(31) and not(addr(6)) ) 
					or   ( kb_data(36) and not(addr(7)) ) );

		z80_db(2) <=		not( ( kb_data(2) and not( addr(0)) ) 
					or   ( kb_data(7) and not( addr(1)) ) 
					or   ( kb_data(12) and not(addr(2)) ) 
					or   ( kb_data(17) and not(addr(3)) ) 
					or   ( kb_data(22) and not(addr(4)) ) 
					or   ( kb_data(27) and not(addr(5)) ) 
					or   ( kb_data(32) and not(addr(6)) ) 
					or   ( kb_data(37) and not(addr(7)) ) );

		z80_db(3) <=		not( ( kb_data(3) and not( addr(0)) ) 
					or   ( kb_data(8) and not( addr(1)) ) 
					or   ( kb_data(13) and not(addr(2)) ) 
					or   ( kb_data(18) and not(addr(3)) ) 
					or   ( kb_data(23) and not(addr(4)) ) 
					or   ( kb_data(28) and not(addr(5)) ) 
					or   ( kb_data(33) and not(addr(6)) ) 
					or   ( kb_data(38) and not(addr(7)) ) );

		z80_db(4) <=		not( ( kb_data(4) and not( addr(0)) ) 
					or   ( kb_data(9) and not( addr(1)) ) 
					or   ( kb_data(14) and not(addr(2)) ) 
					or   ( kb_data(19) and not(addr(3)) ) 
					or   ( kb_data(24) and not(addr(4)) ) 
					or   ( kb_data(29) and not(addr(5)) ) 
					or   ( kb_data(34) and not(addr(6)) ) 
					or   ( kb_data(39) and not(addr(7)) ) );
else
	z80_db <= "ZZZZZ";
end if;

end process;

end Behavioral;
